#! /usr/bin/python2

""" Crusader Kings II Linux Launcher
    2013  Robin C. Thomas <rc.thomas90@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>."""

import os, sys, glob, re, wx, datetime
from subprocess import Popen

# Launcher version
VERSION = "0.1-24012013"

# Console colors
HEADERCOLOR = '\033[95m'
INFOCOLOR = '\033[94m'
OKCOLOR = '\033[92m'
WARNINGCOLOR = '\033[93m'
ERRORCOLOR = '\033[91m'
ENDCOLOR = '\033[0m'

# Configuration file name
CONFIG_FILE = 'ck2launcher.conf'

# Will hold the configuration
CONFIG = {}

# Will contain the Popen object that launches the game
ck2Process = None


# Saves a message to logfile
def log(entry):
  global logfile
  logfile.write('[{0}] {1}\n'.format(str(datetime.datetime.now()), entry))

# Shows a header
def header(text):
  print('{0}{1}{2}'.format(HEADERCOLOR, text, ENDCOLOR))
  log(text)


# Shows info messages
def infoMsg(text):
  print('    {0}{1}{2}'.format(INFOCOLOR, text, ENDCOLOR))
  log(text)
  
  
# Shows warning messages
def warningMsg(text):
  print('    {0}WARNING: {1}{2}'.format(WARNINGCOLOR, text, ENDCOLOR))
  log('WARNING: {0}'.format(text))


# Shows error messages
def errorMsg(text):
  print('    {0}ERROR: {1}{2}'.format(ERRORCOLOR, text, ENDCOLOR))
  log('ERROR: {0}'.format(text))
  
  
# Shows confirmation messages
def okMsg(text):
  print('    {0}{1}{2}'.format(OKCOLOR, text, ENDCOLOR))
  log(text)
  

# Represents a mod
class Mod:
  def __init__(self, filename):
    infoMsg('Found modfile: "{0}".'.format(filename))
    self.filename = filename
    self.getModInfo()
    
    if len(self.directory) > 0:
      # This mod should have a directory to store data in, checking if it exists
      if not os.path.exists(self.directory):
	#No, create it
	os.mkdir(self.directory)
	okMsg('Created data directory for mod "{0}": "{1}"'.format(self.name, self.directory))
    
  
  # Opens the modfile of the current mod and gets it's name and user_dir
  def getModInfo(self):
    global CONFIG
    
    # Try opening the modfile
    try:
      modfile = open(CONFIG['MODPATH'] + '/' + self.filename)
    except IOError:
      # Unable to open modfile
      errorMsg('Unable to load modfile "{0}"! Check permissions.'.format(self.filename))
      exit(1)
    
    moddata = modfile.read()
    
    # Get mod name
    try:
      self.name = re.search('^name[ \t]*=[ \t]*"(.*)"', moddata, re.MULTILINE).group(1)
    except AttributeError:
      # Modname not found
      warningMsg('Could not find mod name for modfile "{0}". Using "{0}" as name.'.format(self.filename))
      self.name = self.filename
      
    # Get mod directory (if available)
    self.directory = re.search('^user_dir[ \t]*=[ \t]*"(.*)"', moddata, re.MULTILINE)
    if (hasattr(self.directory, 'group')):
      self.directory = os.path.dirname(CONFIG['MODPATH']) + '/' + self.directory.group(1)
    else:
      self.directory = ''
    
# END CLASS Mod


# The user interface
class Launcher(wx.Frame):
  def __init__(self, parent, title, mods):
    wx.Frame.__init__(self, parent, title=title, size=(450, 450), style=wx.CAPTION|wx.CLOSE_BOX)
    self.mods = mods
    self.initUI()
    
    
  # Initializes the GUI
  def initUI(self):
    self.panel = wx.Panel(self, -1)
    
    self.box = wx.BoxSizer(wx.VERTICAL)
    
    # The list of mods
    choices = []
    for mod in self.mods:
      choices.append(mod.name)
      
    self.modList = wx.CheckListBox(self.panel, choices=choices, size=(450, -1), style=wx.LC_REPORT|wx.BORDER_SUNKEN)
    
    # Run button
    self.runButton = wx.Button(self.panel, label='Run CK2', size=(450, -1))
    self.runButton.Bind(wx.EVT_BUTTON, self.runButtonClick)
    
    # Add controls to panel
    self.box.Add(self.modList, 6)
    self.box.Add(self.runButton, 1)
    
    self.panel.SetSizer(self.box)
    
    # Center window on screen and show window
    self.Centre()
    self.Show(True)
  
  
  # Event handler for the button click event
  def runButtonClick(self, event):
    global CONFIG
    
    # Get selected mods from list
    selection = self.modList.GetChecked()
    selectedMods = []
    for index in selection:
      selectedMods.append(self.mods[index])
    
    # Prepare command to execute
    command = []
    if len(CONFIG['PREPEND'].strip()) > 0:
      command = CONFIG['PREPEND'].split(' ')
    command.append(CONFIG['GAMEPATH'] + '/ck2')
    
    if (len(selectedMods) == 0):
      # No mods selected, run vanilla game
      okMsg("No mod selected, running vanilla game...")
    else:
      # Append selected mods to command
      okMsg(str(len(selectedMods)) + " mods selected:")
      for mod in selectedMods:
	okMsg('\t' + mod.name + ' (' + mod.filename + ')')
	command.append('-mod=mod/' + mod.filename)
    
    # Execute prepared command
    infoMsg("Running '" + ' '.join(command) + "'")
    global ck2Process
    try:
      ck2Process = Popen(command)
      okMsg('Done. Have fun! :D')
    except OSError:
      # Failure, executable not found
      errorMsg("Unable to run command. Please check that the GAMEPATH is set correctly and that the commands in PREPEND are correct.")
      exit(127)
    
    self.Destroy()
     
    
# END CLASS Launcher


# Detects which mods are available in the mod directory
def detectMods():
  global CONFIG
  
  # Go into mod directory
  try:
    os.chdir(CONFIG['MODPATH'])
  except OSError:
    # The mod directory does not exist
    errorMsg('Could not find mod directory "{0}". Please check configuration.'.format(CONFIG['MODPATH']))
    exit(1)
  
  # Search modfiles
  modfiles = glob.glob("*.mod")
  
  # Create mod object for each found modfile
  mods = []
  for modfile in modfiles:
    mod = Mod(modfile)
    mods.append(mod)
    infoMsg('Found mod "{0}" in file "{1}".'.format(mod.name, mod.filename))
    
  return mods

# End detectMods()


# Reads the configuration from the configuration file
def loadConfiguration():
  global CONFIG
  
  try:
    confFile = open(sys.path[0] + '/' + CONFIG_FILE)
  except IOError:
    errorMsg('Could not open configuration file {0}.'.format(CONFIG_FILE))
    exit(1)
  
  # Line count for error messages
  lineCount = 0
  for line in confFile:
    lineCount += 1
    if len(line) < 2 or line[0] == '#':
      # Empty line or comment, do not analize
      continue;
    
    # All configuration lines should have the following format: VARIABLE = 'value'
    entry = re.search('^([A-Z]*)[ \t]*=[ \t]*[\'"](.*)[\'"]$', line, re.MULTILINE)
    try:
      CONFIG[entry.group(1)] = entry.group(2)
    except AttributeError:
      # This line has no valid format
      errorMsg('Invalid format in configuration file line {0}: "{1}"'.format(lineCount, line).replace('\n', ''))
      exit(1)
      
  # Replace '~' in GAMEPATH with the users homepath
  try:
    CONFIG['GAMEPATH'] = CONFIG['GAMEPATH'].replace('~', os.path.expanduser('~'))
  except KeyError:
    # GAMEPATH is not defined in configuration file
    errorMsg('Required field "GAMEPATH" not defined in configuration file!')
    exit(1)
    
  # Replace '~' in MODPATH with the users homepath
  try:
    CONFIG['MODPATH'] = CONFIG['MODPATH'].replace('~', os.path.expanduser('~'))
  except KeyError:
    # MODPATH is not defined in configuration file
    errorMsg('Required field "MODPATH" not defined in configuration file!')
    exit(1)
    
  if not 'PREPEND' in CONFIG:
    # PREPEND is not defined in configuration file
    errorMsg('Required field "PREPEND" not defined in configuration file!')
    exit(1)
    
# End loadConfiguration()


def main():
  global ck2Process, CONFIG
  
  # Greet user
  header('Crusader Kings 2 Linux Launcher')
  infoMsg('Version {0}'.format(VERSION))
  
  # Load configuration
  loadConfiguration()
  
  # Detect available mods
  okMsg('Detecting mods...')
  mods = detectMods()
  okMsg('Done. Found {0} mods'.format(str(len(mods))))
  
  # Create user interface
  app = wx.App(False)
  launcher = Launcher(None, 'CK2 Launcher ver. {0}'.format(VERSION), mods)
  app.MainLoop()
  
  # If game started wait for process to end
  if ck2Process != None:
    exitCode = ck2Process.wait()
    if exitCode == 0:
      # Game ended correctly
      okMsg('Game closed without error.')
    else:
      #Something went wrong
      errorMsg('Game closed with error code {0}.'.format(str(exitCode)))
      exit(1)
  
  exit(0)
  
# End main()

  
if __name__ == "__main__":
  # Open logfile
  logfile = open('ck2launcher.log', 'a')
  
  main()
  
  
